<?php

// Exit if accessed directly
if (!defined('ABSPATH')) {
    exit;
}

/**
 * Function to check if the calendar prices need updating
 *
 * @param WPBS_Calendar $calendar
 *
 */
function wpbs_pl_update_calendar_prices_cron(WPBS_Calendar $calendar)
{

    if (defined('DOING_AJAX') && DOING_AJAX) {
        return;
    }

    if (!wpbs_is_pricing_enabled()) {
        return;
    }

    // Get the settings
    $settings = get_option('wpbs_settings', array());

    if (empty($settings['payment_pricelabs_api'])) {
        return;
    }

    // Get and set refresh time
    if (empty($settings['payment_pricelabs_refresh']) || $settings['payment_pricelabs_refresh'] == 'daily') {
        $refresh_time = DAY_IN_SECONDS;
    } else {
        if ($settings['payment_pricelabs_refresh'] == 'twice_per_day') {
            $refresh_time = DAY_IN_SECONDS / 2;
        } elseif ($settings['payment_pricelabs_refresh'] == 'hourly') {
            $refresh_time = HOUR_IN_SECONDS;
        }
    }

    $pricelabs_id = wpbs_get_calendar_meta($calendar->get('id'), 'pricelabs_id', true);

    if (!$pricelabs_id) {
        return;
    }

    $last_updated = wpbs_get_calendar_meta($calendar->get('id'), 'pricelabs_last_updated_' . $pricelabs_id, true);

    if ($refresh_time != 0 && strtotime($last_updated) > (current_time('timestamp') - $refresh_time)) {
        return;
    }

    wpbs_pl_update_calendar_prices($calendar->get('id'));
}

add_action('wpbs_calendar_outputter', 'wpbs_pl_update_calendar_prices_cron');
add_action('wpbs_overview_calendar_outputter_calendar', 'wpbs_pl_update_calendar_prices_cron');

/**
 * Update calendar prices
 *
 * @param int $calendar_id
 *
 */
function wpbs_pl_update_calendar_prices($calendar_id)
{

    $pricelabs = new WPBS_PriceLabs_API();

    if (!$pricelabs->validate_api_key()) {
        return;
    }

    $pricelabs_id = wpbs_get_calendar_meta($calendar_id, 'pricelabs_id', true);

    if (!$pricelabs_id) {
        return;
    }

    $pricelabs_pms = wpbs_get_calendar_meta($calendar_id, 'pricelabs_pms', true);

    if (!$pricelabs_pms) {
        return;
    }

    // Get the settings
    $settings = get_option('wpbs_settings', array());

    $discount = (isset($settings['payment_pricelabs_discount']) && !empty($settings['payment_pricelabs_discount'])) ? floatval($settings['payment_pricelabs_discount']) : false;

    $prices = $pricelabs->get_listing_price($pricelabs_id, $pricelabs_pms);

    if (isset($prices->data)) foreach ($prices->data as $price) {
        $date = DateTime::createFromFormat('Y-m-d', $price->date);

        $price_value = $price->price;

        if ($discount) {
            $price_value = $price_value - (($price_value / 100) * $discount);
        }

        $price_value = apply_filters('wpbs_pricelabs_price', $price_value);

        $event_data = [
            'calendar_id' => $calendar_id,
            'date_day' => $date->format('d'),
            'date_month' => $date->format('m'),
            'date_year' => $date->format('Y'),
            'price' => $price_value,
        ];

        $event = wpbs_get_events(array('calendar_id' => $calendar_id, 'date_day' => $event_data['date_day'], 'date_month' => $event_data['date_month'], 'date_year' => $event_data['date_year']));

        if (!empty($event[0])) {
            wpbs_update_event($event[0]->get('id'), $event_data);
            $event_id = $event[0]->get('id');
        } else {
            $event_id = wpbs_insert_event($event_data);
        }

        wpbs_update_event_meta($event_id, 'pricelabs_min_stay', $price->min_stay);
    }

    wpbs_update_calendar_meta($calendar_id, 'pricelabs_last_updated_' . $pricelabs_id, current_time('Y-m-d H:i:s'));
}


/**
 * Function that hooks into the main WPBS_Form_Validator Class
 * 
 * @param array $output
 * @param WPBS_Form $form
 * @param array $form_args
 * @param WPBS_Calendar $calendar
 * @param string $language
 * 
 * @return array
 * 
 */
function wpbs_pl_validate_form($output, $form, $form_args, $calendar, $calendar_args, $language)
{
    $error = false;

    $pricelabs_id = wpbs_get_calendar_meta($calendar->get('id'), 'pricelabs_id', true);
    $pricelabs_pms = wpbs_get_calendar_meta($calendar->get('id'), 'pricelabs_pms', true);

    if (!$pricelabs_id) {
        return $output;
    }


    if (!$pricelabs_pms) {
        return $output;
    }

    // Check if validation is enabled
    $pricelabs_restrictions = wpbs_get_calendar_meta($calendar->get('id'), 'pricelabs_restrictions', true);
    if ($pricelabs_restrictions) {
        return $output;
    }

    // Get pricelabs min stay
    $events = wpbs_get_events(array('calendar_id' => $calendar->get('id'), 'date_year' => date('Y', $calendar_args['start_date']), 'date_month' => date('m', $calendar_args['start_date']), 'date_day' => date('d', $calendar_args['start_date'])));

    if (!isset($events[0])) {
        return $output;
    }

    $minimum_stay = absint(wpbs_get_event_meta($events[0]->get('id'), 'pricelabs_min_stay', true));

    if ($minimum_stay === 0) {
        return $output;
    }

    // Get the number of selected days
    $number_of_selected_days = ($calendar_args['end_date'] - $calendar_args['start_date']) / DAY_IN_SECONDS;
    if ($form_args['selection_style'] != 'split') {
        $number_of_selected_days += 1;
    }

    if ($number_of_selected_days < $minimum_stay) {
        $error = true;
        $error_message = sprintf(wpbs_get_form_default_string($form->get('id'), 'minimum_selection', $language), $minimum_stay);
    }

    if ($error === true) {

        return array(
            'form_args' => $form_args,
            'calendar_args' => $calendar_args,
            'error_message' => $error_message,
            'error' => $error,
        );
    }

    return $output;
}
add_filter('wpbs_form_validator_custom_validation', 'wpbs_pl_validate_form', 30, 6);
